function Fashion_ResetData()
	FashionData = {
		helm = {
			show = {},
			hide = {},
		},
		cloak = {
			show = {},
			hide = {},
		},
	}
end


-- check if your helm is supposed to be shown or hidden every time your inventory changes
function Fashion_RefreshView()
	local helmLink = GetInventoryItemLink("player", GetInventorySlotInfo("HeadSlot"))
	local cloakLink = GetInventoryItemLink("player", GetInventorySlotInfo("BackSlot"))
	
	local helmShown, cloakShown
	
	if helmLink then
		helmShown = Fashion_SearchLists(helmLink:match("item:(%d+)"), "helm")
	end
	if cloakLink then
		cloakShown = Fashion_SearchLists(cloakLink:match("item:(%d+)"), "cloak")
	end
	
	local shown = {
		show = true,
		hide = false,
	}
	
	-- show/hide depending on which list item was in
	if helmShown then
		ShowHelm(shown[helmShown])
	end
	if cloakShown then
		ShowCloak(shown[cloakShown])
	end
end


-- search the tables for an item
function Fashion_SearchLists(itemID, equipLoc)
	local foundList, foundKey
	
	for k, v in pairs(FashionData[equipLoc]) do
		for index, item in ipairs(v) do
			if item == tonumber(itemID) then
				-- if found, return index and which list
				foundKey = index
				foundList = k
				break
			end
		end
	end
	
	return foundList, foundKey
end


function Fashion_OnEvent(self, event, ...)
	if event == "UNIT_INVENTORY_CHANGED" then
		if select(1, ...) == "player" then
			-- refresh helm/cloak visibility every time your inventory changes
			Fashion_RefreshView()
		end
	elseif select(1, ...) == "Fashion" then
		-- initiate variables if none existing
		if FashionData then
			if not FashionData.helm then
				-- convert data from previous version (1.2)
				FashionData = {
					helm = {
						show = {},
						hide = {},
					},
					cloak = {
						show = {},
						hide = {},
					},
				}
				FashionData.show = nil
				FashionData.hide = nil
			end
		else
			Fashion_ResetData()
		end
	end
end


function Fashion_SlashHandler(msg)
	local arg = {}
	for v in gmatch(strlower(msg), "[^ ]+") do
		arg[#arg + 1] = v
	end
	-- local cmd, arg = string.split(" ", msg or "")
	local cmd = arg[1]
	local arg1 = arg[2]
	local arg2 = arg[3]
	local itemID, link
	
	if arg1 then
		-- if the provided argument was a valid item link we should be able to extract the item ID
		itemID = arg1:match("item:(%d+)")
		if itemID then
			link = select(2, GetItemInfo(itemID))
		end
	end
	
	local equipSlots = {
		INVTYPE_HEAD = "helm",
		INVTYPE_CLOAK = "cloak",
	}
	
	if cmd == "show" or cmd == "hide" then
		if itemID then
			local equipLoc = equipSlots[select(9, GetItemInfo(itemID))]
			
			-- proceed only if the provided link is a head or back item
			if equipLoc then
				local foundList, foundKey = Fashion_SearchLists(itemID, equipLoc)
				
				local otherList
				if cmd == "show" then
					otherList = "hide"
				else
					otherList = "show"
				end
				
				-- was this item already on the table? abort
				if foundList == cmd then
					print(link.." is already on "..cmd.." list.")
					return
				-- or maybe on the other table? remove it from there
				elseif foundList == otherList then
					FashionData[equipLoc][otherList][foundKey] = nil
					print(link.." removed from "..otherList.." list.")
				end
				
				-- add the item to the table
				tinsert(FashionData[equipLoc][cmd], tonumber(itemID))
				print(link.." added to "..cmd.." list.")
			else
				print("Invalid equip location. Please provide a head or cloak item link.")
			end
			Fashion_RefreshView()
		else
			print("Invalid argument to '"..cmd.."'. Please provide a valid item link.")
		end
	elseif cmd == "clear" then
		-- an item link is provided, reset its eventual status
		if itemID then
			local equipLoc = equipSlots[select(9, GetItemInfo(itemID))]
			
			if equipLoc then
				local foundList, foundKey = Fashion_SearchLists(itemID, equipLoc)
				
				-- remove this item from the table it was found on, if it was
				if foundList then
					FashionData[equipLoc][foundList][foundKey] = nil
					print(link.." removed from "..foundList.." list.")
				else
					print(link.." was not found on any list.")
				end
				
				Fashion_RefreshView()
			else
				print("Invalid equip location. Please provide a head or back item link.")
			end
		-- a list is provided, reset it
		elseif arg1 == "helm" or arg1 == "cloak" then
			local output = strupper(strsub(arg1, 1, 1))..strsub(arg1, 2)
			if arg2 == "show" or arg2 == "hide" then
				FashionData[arg1][arg2] = {}
				output = output.." "..arg2
			elseif arg2 then
				print("Invalid argument to 'clearlist "..arg1.."'. Please choose to clear show or hide list, or none to clear both.")
				return
			else
				FashionData[arg1] = {
					show = {},
					hide = {},
				}
			end
			print(output.." list cleared.")
		elseif arg1 then
			print("Invalid argument to 'clear'. Please provide a valid item link, a list name (helm/cloak) and, optionally, sublist name (show/hide) or nothing to reset all items.")
			-- return
		else
			Fashion_ResetData()
			print("All items reset.")
		end
	else
		-- if no valid slash command was provided; print help dialogue
		print(" How to use Fashion:")
		print("/fashion [show | hide] |cff1eff00[itemLink]|r - Add an item to be shown or hidden.")
		print("/fashion clear |cff1eff00[itemLink]|r - Resets the specified item.")
		print("/fashion clear [helm | cloak [show | hide]] - Clears the specified list. Leave out parameters to clear all sublists.")
	end
end


SlashCmdList["FASHION"] = Fashion_SlashHandler
SLASH_FASHION1 = "/fashion"


local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", Fashion_OnEvent)
frame:RegisterEvent("ADDON_LOADED")
frame:RegisterEvent("UNIT_INVENTORY_CHANGED")